@@ -4,9 +4,9 @@ import errno
 import os
 import sys
 
-from PySide.QtCore import Qt, QCoreApplication, QDir, QFile, QIODevice, \
+from PyQt4.QtCore  import Qt, QCoreApplication, QDir, QFile, QIODevice, \
                           QString, QStringList, QTextStream, QVariant
-from PySide.QtSql  import QSqlDatabase, QSqlQuery
+from PyQt4.QtSql   import QSqlDatabase, QSqlQuery
 
 class DBException(Exception): #{{{1
     def __init__(self, errmsg, dberr):
@@ -34,6 +34,7 @@ class DBExceptionData(Exception): #{{{1
 class LTDatabase(): #{{{1
     # Class variables {{{2
     _Conn        = None
+    _colNames    = {}
 
     # Define the configuration directory, and the database filename #{{{2
     _configDir = '.pybraryThing'
@@ -143,6 +144,8 @@ class LTDatabase(): #{{{1
         if not tables.contains(QString('display_filters')):
             self._createTableDisplayFilters()
 
+        self._colNames = self.getColumnNames()
+
     def _createTableDisplayOptions(self): #{{{2
         # Create the display options table
         q = QSqlQuery(self.SQL_CREATE_DISPLAY_OPTIONS, self._conn)
@@ -260,6 +263,9 @@ class LTDatabase(): #{{{1
             if progress != None:
                 rowNum += 1
                 progress.setValue(rowNum)
+                if progress.wasCanceled():
+                    self._conn.rollback()
+                    break
 
             # Skip the header line
             if not seenHeader:
@@ -279,22 +285,22 @@ class LTDatabase(): #{{{1
                 val = None
                 insertTags = False
 
-                if row.size() == 28 and i > 21:
+                if len(row) == 28 and i > 21:
                     i -= 1
 
                 if fieldName == 'isbn':
-                    isbn = row.at(i)
+                    isbn = row[i]
                     while isbn.startsWith('['):
                         isbn = isbn.mid(1)
                     while isbn.endsWith(']'):
                         isbn.chop(1)
                     val = QVariant(isbn)
-                elif fieldName == 'tags' and row.size() == 28:
+                elif fieldName == 'tags' and len(row) == 28:
                     val = QVariant(QVariant.String)
                 elif row[i] == '(blank)' or row[i] == '':
                     val = QVariant(QVariant.String)
                 else:
-                    val = QVariant(row.at(i))
+                    val = QVariant(row[i])
 
                 q.bindValue(":%s" % fieldName, val)
 
@@ -355,7 +361,7 @@ class LTDatabase(): #{{{1
             raise DBExceptionExecute(q)
 
         while q.next():
-            res = q.result().data(0).toString()
+            res = q.record().value(0).toString()
             querySql += res + ', '
 
         querySql.chop(2)
@@ -374,7 +380,7 @@ class LTDatabase(): #{{{1
         sortSeen = False
         while q.next():
             sortSeen = True
-            res = q.result().data(0).toString()
+            res = q.record().value(0).toString()
             querySql += 'upper(' + res + '), '
 
         if sortSeen:
@@ -397,10 +403,10 @@ class LTDatabase(): #{{{1
         i = 0
         while q.next():
             filterSeen = True
-            res = q.result()
-            col = res.data(0).toString()
-            method = res.data(1).toString()
-            f_string = res.data(2).toString()
+            res = q.record()
+            col = res.value(0).toString()
+            method = res.value(1).toString()
+            f_string = res.value(2).toString()
 
             if method == '=':
                 querySql += 'upper(%s) = upper(:%d) and ' % (col, i)
@@ -486,7 +492,7 @@ class LTDatabase(): #{{{1
 
         cols = QStringList()
         while q.next():
-            cols.append(q.result().data(0).toString())
+            cols.append(q.record().value(0).toString())
 
         return cols
 
@@ -538,6 +544,15 @@ class LTDatabase(): #{{{1
         self._conn.commit()
 
     def getFieldNames(self): #{{{2
+        colNames = self.getColumnNames()
+        fieldNames = dict([(v, k) for (k, v) in colNames.iteritems()])
+
+        return fieldNames
+
+    def getColumnNames(self): #{{{2
+        if len(self._colNames) > 0:
+            return self._colNames
+
         # Get the column names
         q = QSqlQuery(self.SQL_SELECT_FIELD_NAMES, self._conn)
         if not q.isActive():
@@ -545,7 +560,7 @@ class LTDatabase(): #{{{1
 
         fieldNames = {}
         while q.next():
-            fieldNames[q.value(0).toString()] = q.value(1).toString()
+            fieldNames[q.value(1).toString()] = q.value(0).toString()
 
         return fieldNames
 
@@ -645,6 +660,21 @@ class LTDatabase(): #{{{1
     def filterList(self): #{{{2
         return self.getColumns(self.SQL_FILTER_LIST)
 
+    def hasContent(self, coldesc): #{{{2
+        map = self.getColumnNames()
+        colname = map[coldesc]
+        query = "select 1 from books where %s is not null and %s != ''" % (colname, colname)
+
+        q = QSqlQuery(query, self._conn)
+        if not q.isActive():
+            raise DBExceptionExecute(q)
+
+        if q.next():
+            q.finish()
+            return True
+
+        return False
+
 # Handle running as a script {{{1
 if __name__ == '__main__':
     app = QCoreApplication(sys.argv)
