@@ -1,19 +1,20 @@
 #!/usr/bin/env python
 
-from PySide.QtCore    import Qt, QByteArray, QIODevice, QModelIndex, QString, \
-                             QTextStream, QUrl, QVariant, SIGNAL
-from PySide.QtGui     import QAbstractItemView, QAction, QComboBox, QDialog, \
+from PyQt4.QtCore     import Qt, QByteArray, QIODevice, QModelIndex, QObject, \
+                             QString, QStringList, QTextStream, QUrl, \
+                             QVariant, SIGNAL
+from PyQt4.QtGui      import QAbstractItemView, QAction, QComboBox, QDialog, \
                              QDialogButtonBox, QFileDialog, QGridLayout, \
                              QGroupBox, QHBoxLayout, QInputDialog, QLabel, \
                              QLineEdit, QListView, QMenuBar, QMessageBox, \
                              QProgressDialog, QPushButton, QStringListModel, \
                              QTableView, QToolButton, QVBoxLayout, QWidget
-from PySide.QtSql     import QSqlQueryModel
-from PySide.QtNetwork import QNetworkAccessManager, QNetworkCookieJar, \
+from PyQt4.QtSql      import QSqlQueryModel
+from PyQt4.QtNetwork  import QNetworkAccessManager, QNetworkCookieJar, \
                              QNetworkReply, QNetworkRequest
 from LTdatabase       import LTDatabase
 
-version = '0.0.3'
+version = '0.0.4'
 exportUrl = QUrl('http://www.librarything.com/export-tab')
 loginUrl  = QUrl('http://www.librarything.com/signup.php')
 
@@ -42,23 +43,28 @@ class LTFilterDefine(QDialog): #{{{1
         self._filters.setSelectionMode(QAbstractItemView.NoSelection)
 
         action = QAction('Remove Rule', self._filters)
-        action.connect(SIGNAL('triggered(bool)'), self.removeRule)
+        QObject.connect(action, SIGNAL('triggered(bool)'), self.removeRule)
         self._filters.addAction(action)
         self._filters.setContextMenuPolicy(Qt.ActionsContextMenu)
 
         # Set up the list of columns for filtering on {{{3
-        fieldNames   = database.getFieldNames()
-        self._revFieldNames = dict([(v, k) for (k, v) in fieldNames.iteritems()])
+        self._colNames = database.getColumnNames()
+
+        displayNamesTmp = self._colNames.keys()
+        displayNames = QStringList()
+
+        for col in displayNamesTmp:
+            if self._database.hasContent(col):
+                displayNames.append(col)
 
-        displayNames = self._revFieldNames.keys()
         displayNames.sort()
 
         self._filterFields = QComboBox()
         for val in displayNames:
-            if self._revFieldNames[val] == 'id':
+            if self._colNames[val] == 'id':
                 continue
 
-            self._filterFields.addItem(val, QVariant(self._revFieldNames[val]))
+            self._filterFields.addItem(val, QVariant(self._colNames[val]))
 
         # Set up the list of filtering methods {{{3
         self._filterMethods = QComboBox()
@@ -82,12 +88,12 @@ class LTFilterDefine(QDialog): #{{{1
         filterBtns.addButton(btnSave, QDialogButtonBox.ActionRole)
         filterBtns.addButton(btnClear, QDialogButtonBox.ResetRole)
 
-        filterBtns.connect(SIGNAL('accepted()'), self.applyFilter)
-        filterBtns.connect(SIGNAL('rejected()'), self.cancelFilter)
-        btnClear.connect(SIGNAL('clicked(bool)'), self.clearFilter)
-        btnAdd.connect(SIGNAL('clicked(bool)'), self.addRule)
-        btnLoad.connect(SIGNAL('clicked(bool)'), self.loadFilter)
-        btnSave.connect(SIGNAL('clicked(bool)'), self.saveFilter)
+        QObject.connect(filterBtns, SIGNAL('accepted()'), self.applyFilter)
+        QObject.connect(filterBtns, SIGNAL('rejected()'), self.cancelFilter)
+        QObject.connect(btnClear, SIGNAL('clicked(bool)'), self.clearFilter)
+        QObject.connect(btnAdd, SIGNAL('clicked(bool)'), self.addRule)
+        QObject.connect(btnLoad, SIGNAL('clicked(bool)'), self.loadFilter)
+        QObject.connect(btnSave, SIGNAL('clicked(bool)'), self.saveFilter)
 
         # Layout all the elements {{{3
         hbox1 = QHBoxLayout()
@@ -133,7 +139,7 @@ class LTFilterDefine(QDialog): #{{{1
 
         model = self._filters.model()
 
-        filterField = self._revFieldNames[model.index(ruleRow, 0).data().toString()]
+        filterField = self._colNames[model.index(ruleRow, 0).data().toString()]
         filterMethod = model.index(ruleRow, 1).data().toString()
         filterString = model.index(ruleRow, 2).data().toString()
         
@@ -179,7 +185,7 @@ class LTColumnSelectGeneric(QDialog): #{{{1
         self._availableColumns.setModel(model)
         self._availableColumns.setUniformItemSizes(True)
         self._availableColumns.setSelectionMode(QAbstractItemView.SingleSelection)
-        self._availableColumns.connect(SIGNAL('doubleClicked(QModelIndex)'), self.moveItemClick)
+        QObject.connect(self._availableColumns, SIGNAL('doubleClicked(QModelIndex)'), self.moveItemClick)
         self._availableColumns.model().sort(0)
 
         lyt = QVBoxLayout()
@@ -193,7 +199,7 @@ class LTColumnSelectGeneric(QDialog): #{{{1
         self._selectedColumns.setModel(model)
         self._selectedColumns.setUniformItemSizes(True)
         self._selectedColumns.setSelectionMode(QAbstractItemView.SingleSelection)
-        self._selectedColumns.connect(SIGNAL('doubleClicked(QModelIndex)'), self.moveItemClick)
+        QObject.connect(self._selectedColumns, SIGNAL('doubleClicked(QModelIndex)'), self.moveItemClick)
         self._selectedColumns.model() # This is needed to force the display
 
         lyt = QVBoxLayout()
@@ -206,11 +212,11 @@ class LTColumnSelectGeneric(QDialog): #{{{1
         # Set up the up/down arrows for ordering the display columns {{{3
         orderBtnUp = QToolButton()
         orderBtnUp.setArrowType(Qt.UpArrow)
-        orderBtnUp.connect(SIGNAL('clicked(bool)'), self.orderItemUp)
+        QObject.connect(orderBtnUp, SIGNAL('clicked(bool)'), self.orderItemUp)
 
         orderBtnDown = QToolButton()
         orderBtnDown.setArrowType(Qt.DownArrow)
-        orderBtnDown.connect(SIGNAL('clicked(bool)'), self.orderItemDown)
+        QObject.connect(orderBtnDown, SIGNAL('clicked(bool)'), self.orderItemDown)
 
         orderBtnBox = QVBoxLayout()
         orderBtnBox.addWidget(orderBtnUp)
@@ -219,11 +225,11 @@ class LTColumnSelectGeneric(QDialog): #{{{1
         # Set up the left/right arrows for moving to/from display {{{3
         selectBtnRight = QToolButton()
         selectBtnRight.setArrowType(Qt.RightArrow)
-        selectBtnRight.connect(SIGNAL('clicked()'), self.moveItemRight)
+        QObject.connect(selectBtnRight, SIGNAL('clicked()'), self.moveItemRight)
 
         selectBtnLeft = QToolButton()
         selectBtnLeft.setArrowType(Qt.LeftArrow)
-        selectBtnLeft.connect(SIGNAL('clicked()'), self.moveItemLeft)
+        QObject.connect(selectBtnLeft, SIGNAL('clicked()'), self.moveItemLeft)
 
         selectBtnBox = QVBoxLayout()
         selectBtnBox.addWidget(selectBtnRight)
@@ -231,8 +237,8 @@ class LTColumnSelectGeneric(QDialog): #{{{1
 
         # Add the Save/Cancel buttons #{{{3
         exitButtons = QDialogButtonBox(QDialogButtonBox.Save|QDialogButtonBox.Cancel,Qt.Horizontal)
-        exitButtons.connect(SIGNAL('accepted()'), self.saveSettings)
-        exitButtons.connect(SIGNAL('rejected()'), self.cancelSelection)
+        QObject.connect(exitButtons, SIGNAL('accepted()'), self.saveSettings)
+        QObject.connect(exitButtons, SIGNAL('rejected()'), self.cancelSelection)
 
         # Add all the widgets to the window {{{3
         selLyt.addWidget(exitButtons)
@@ -312,13 +318,22 @@ class LTColumnSelectGeneric(QDialog): #{{{1
     def cancelSelection(self): #{{{2
         self.reject()
 
+    def checkColumns(self, list): #{{{2
+        new_list = QStringList()
+
+        for col in list:
+            if self._database.hasContent(col):
+                new_list.append(col)
+
+        return new_list
+
 class LTColumnSelectDisplay(LTColumnSelectGeneric): #{{{1
     def __init__(self, parent, database): #{{{2
         LTColumnSelectGeneric.__init__(self, parent, database)
 
         self.setWindowTitle('Select and order columns to display')
 
-        availableModel = QStringListModel(database.getNonDisplayColumns())
+        availableModel = QStringListModel(self.checkColumns(database.getNonDisplayColumns()))
         selectedModel  = QStringListModel(database.getDisplayColumns())
 
         availableList = self.setupAvailableList(availableModel)
@@ -339,7 +354,7 @@ class LTColumnSelectSort(LTColumnSelectGeneric): #{{{1
 
         self.setWindowTitle('Select and order columns to sort by')
 
-        availableModel = QStringListModel(database.getNonSortColumns())
+        availableModel = QStringListModel(self.checkColumns(database.getNonSortColumns()))
         selectedModel  = QStringListModel(database.getSortColumns())
 
         availableList = self.setupAvailableList(availableModel)
@@ -422,9 +437,19 @@ class LTAboutWindow(QDialog): #{{{1
         lyt.addWidget(txt)
 
 class LTMainWindow(QWidget): #{{{1
+    _maemo = False
+    _downloadCompleteSet = False
+
     def __init__(self): #{{{2
         QWidget.__init__(self, None)
 
+        try:
+            from PyQt4.QtMaemo5 import QMaemo5InformationBox
+            self._maemo = True
+            self.setAttribute(Qt.WA_Maemo5ShowProgressIndicator, True)
+        except:
+            pass
+
         # Set to the N900 screen size to aid testing
         self.setMinimumSize(800, 480)
         self.setWindowTitle('LibraryThing Viewer')
@@ -435,12 +460,18 @@ class LTMainWindow(QWidget): #{{{1
         menu = QMenuBar()
 
         # Set up the menu
-        menu.addAction('Display Fields').connect(SIGNAL('triggered()'), self.displayFields)
-        menu.addAction('Sort Order').connect(SIGNAL('triggered()'), self.sortOrder)
-        menu.addAction('Filters').connect(SIGNAL('triggered()'), self.displayFilters)
-        menu.addAction('Load Export File').connect(SIGNAL('triggered()'), self.loadFile)
-        menu.addAction('Download Export File').connect(SIGNAL('triggered()'), self.downloadExportFile)
-        menu.addAction('About').connect(SIGNAL('triggered()'), self.about)
+        act = menu.addAction('Display Fields')
+        QObject.connect(act, SIGNAL('triggered()'), self.displayFields)
+        act = menu.addAction('Sort Order')
+        QObject.connect(act, SIGNAL('triggered()'), self.sortOrder)
+        act = menu.addAction('Filters')
+        QObject.connect(act, SIGNAL('triggered()'), self.displayFilters)
+        act = menu.addAction('Load Export File')
+        QObject.connect(act, SIGNAL('triggered()'), self.loadFile)
+        act = menu.addAction('Download Export File')
+        QObject.connect(act, SIGNAL('triggered()'), self.downloadExportFile)
+        act = menu.addAction('About')
+        QObject.connect(act, SIGNAL('triggered()'), self.about)
 
         self.layout().setMenuBar(menu)
         self._menu = menu
@@ -460,9 +491,8 @@ class LTMainWindow(QWidget): #{{{1
         # Set a cookie jar
         self._network.setCookieJar(QNetworkCookieJar())
 
-        # Handle network responses - this does not work currently
-        # Instead connect the signal to each request
-#        self._network.connect(SIGNAL('finished(QNetworkReply)'), self.replyFinished)
+        # Handle network responses
+        QObject.connect(self._network, SIGNAL('finished(QNetworkReply *)'), self.replyFinished)
 
         # Set up the view pane
         self._tab.setCornerButtonEnabled(False)
@@ -472,17 +502,34 @@ class LTMainWindow(QWidget): #{{{1
         self._tab.horizontalHeader().setMovable(True)
 
         # Handle moving/resizing of columns
-        self._tab.horizontalHeader().connect(SIGNAL('sectionMoved(int, int, int)'), self.changeColumnOrder)
-        self._tab.horizontalHeader().connect(SIGNAL('sectionResized(int, int, int)'), self.changeColumnSize)
-        self._tab.horizontalHeader().connect(SIGNAL('sectionDoubleClicked(int)'), self._tab.resizeColumnToContents)
+        QObject.connect(self._tab.horizontalHeader(), SIGNAL('sectionMoved(int, int, int)'), self.changeColumnOrder)
+        QObject.connect(self._tab.horizontalHeader(), SIGNAL('sectionResized(int, int, int)'), self.changeColumnSize)
+        QObject.connect(self._tab.horizontalHeader(), SIGNAL('sectionDoubleClicked(int)'), self._tab.resizeColumnToContents)
+
+        # No way to get from the QVariant to a QAbstractKineticScroller
+        # Fiddle with scrolling
+#       scroller = self._tab.property('kineticScroller')
+
+#       if scroller.isValid():
+#           scroller.setLowFrictionEnabled(True)
+#           scroller.setAxisLockThreshold('0.5')
 
         # Handle double-clicking an entry
-        self._tab.connect(SIGNAL('doubleClicked(QModelIndex)'), self.viewBookInfo)
+        QObject.connect(self._tab, SIGNAL('doubleClicked(QModelIndex)'), self.viewBookInfo)
                     
         # Set up the main view
         self.layout().addWidget(self._tab)
 
+        # Clear the progress indicator
+        if self._maemo:
+            self.setAttribute(Qt.WA_Maemo5ShowProgressIndicator, False)
+
     def loadData(self): #{{{2
+        updateSpinner = False
+        if self._maemo and not self.testAttribute(Qt.WA_Maemo5ShowProgressIndicator):
+            updateSpinner = True
+            self.setAttribute(Qt.WA_Maemo5ShowProgressIndicator, True)
+
         # Set the SQL query to use
         self._database.getQuery(self._tab.model())
 
@@ -514,6 +561,9 @@ class LTMainWindow(QWidget): #{{{1
                 self._tab.setColumnWidth(i + 1, width)
                 self.changeColumnSize(i + 1, 0, width)
 
+        if updateSpinner:
+            self.setAttribute(Qt.WA_Maemo5ShowProgressIndicator, False)
+
     def displayFields(self): #{{{2
         winFields = LTColumnSelectDisplay(self, self._database)
         if winFields.exec_():
@@ -528,10 +578,14 @@ class LTMainWindow(QWidget): #{{{1
         filename = QFileDialog.getOpenFileName(self, 'Open LT Export')
 
         if filename != None:
-            self.setupProgressBar('LoadingData')
+            if self._maemo:
+                self.setAttribute(Qt.WA_Maemo5ShowProgressIndicator, True)
+            self.setupProgressBar('Loading data')
             self._database.loadFile(filename, self._progress)
-            self.__tab.model().clear()
+            self._tab.model().clear()
             self.loadData()
+            if self._maemo:
+                self.setAttribute(Qt.WA_Maemo5ShowProgressIndicator, False)
 
     def loginLT(self): #{{{2
         # TODO: Move the username/password to a config setting
@@ -549,7 +603,6 @@ class LTMainWindow(QWidget): #{{{1
         # Login
         post_data =  'formusername=%s&formpassword=%s&Submit=Sign+in' % (userName, password)
         reply = self._network.post(QNetworkRequest(loginUrl), QByteArray(post_data))
-        reply.connect(SIGNAL('finished()'), self.replyFinished)
 
         self._loginBox = QDialog(self)
         self._loginBox.setWindowTitle('Logging in')
@@ -558,16 +611,23 @@ class LTMainWindow(QWidget): #{{{1
         self._loginBox.exec_()
 
     def downloadExportFile(self): #{{{2
-        self.disconnect(SIGNAL('downloadComplete()'), self.downloadExportFile)
+        if self._downloadCompleteSet:
+            QObject.disconnect(self, SIGNAL('downloadComplete()'), self.downloadExportFile)
 
         if self.haveLTCookie():
-            self.setupProgressBar('Downloading file')
-            reply = self._network.get(QNetworkRequest(exportUrl))
-            reply.connect(SIGNAL('downloadProgress(qint64, qint64)'), self.downloadProgress)
+            if self._maemo:
+                self.setAttribute(Qt.WA_Maemo5ShowProgressIndicator, True)
 
-            reply.connect(SIGNAL('finished()'), self.replyFinished)
+            reply = self._network.get(QNetworkRequest(exportUrl))
+
+            self._loginBox = QDialog(self)
+            self._loginBox.setWindowTitle('Downloading')
+            lyt = QHBoxLayout(self._loginBox)
+            lyt.addWidget(QLabel('Downloading LibraryThing export file'))
+            self._loginBox.exec_()
         else:
-            self.connect(SIGNAL('downloadComplete()'), self.downloadExportFile)
+            QObject.connect(self, SIGNAL('downloadComplete()'), self.downloadExportFile)
+            self._downloadCompleteSet = True
             self.loginLT()
 
     def haveLTCookie(self): #{{{2
@@ -579,12 +639,8 @@ class LTMainWindow(QWidget): #{{{1
 
     def setupProgressBar(self, title): #{{{2
         self._progress.setWindowTitle(title)
-        self._progress.setMaximum(100)
-        self._progress.setValue(1)
-
-    def downloadProgress(self, done, total): #{{{2
-        self._progress.setMaximum(total)
-        self._progress.setValue(done)
+        self._progress.setMaximum(0)
+        self._progress.setValue(0)
 
     def about(self): #{{{2
         aboutWin = LTAboutWindow(self)
@@ -615,11 +671,11 @@ class LTMainWindow(QWidget): #{{{1
 
                 req = QNetworkRequest(url)
                 rep = self._network.get(req)
-                rep.connect(SIGNAL('finished()'), self.replyFinished)
             elif reply.url() == exportUrl:
+                self._loginBox.done(0)
                 export = reply.readAll()
                 reply.deleteLater()
-                self.emit(SIGNAL('downloadComplete()'))
+                QObject.emit(self, SIGNAL('downloadComplete()'))
 
                 stream = QTextStream(export, QIODevice.ReadOnly)
 
@@ -627,20 +683,24 @@ class LTMainWindow(QWidget): #{{{1
                 self._database.loadStream(stream, self._progress)
                 self._tab.model().clear()
                 self.loadData()
+
+                if self._maemo:
+                    self.setAttribute(Qt.WA_Maemo5ShowProgressIndicator, False)
             else:
                 self._loginBox.done(0)
                 res = reply.readAll()
                 reply.deleteLater()
 
                 if res.indexOf('Wrong user name or password') != -1:
-                    self.disconnect(SIGNAL('downloadComplete()'), self.downloadExportFile)
+                    QObject.disconnect(self, SIGNAL('downloadComplete()'), self.downloadExportFile)
+                    self._downloadCompleteSet = False
                     QMessageBox.critical(self, 'Login failed', 'Username or password incorrect')
 
-                self.emit(SIGNAL('downloadComplete()'))
+                QObject.emit(self, SIGNAL('downloadComplete()'))
         else:
             QMessageBox.critical(self, 'Network error', reply.errorString())
             reply.deleteLater()
-            self.emit(SIGNAL('downloadComplete()'))
+            QObject.emit(self, SIGNAL('downloadComplete()'))
 
     def displayFilters(self): #{{{2
         filterWin = LTFilterDefine(self, self._database)
