@@ -3,7 +3,7 @@
 import sys
 import traceback
 
-from PySide.QtGui import QApplication, QMessageBox
+from PyQt4.QtGui import QApplication, QMessageBox
 from LTgui import LTMainWindow
 
 if __name__ == '__main__':
